#include <iostream>
#include <stdlib.h>

#include "sqldb.h"
#include "strmanip.h"

using std::endl;
using std::cerr;

SqlDb::SqlDb() : nrow(0), ncol(0), errmsg(0)
{
    char *homedir = getenv("HOME");
    string dbpath = homedir;    
    db = sqlite_open(dbpath.append("/.imms.db").c_str(), 600, &errmsg);

    sql_create_table();
}

void SqlDb::sql_create_table()
{
    string create_table_query = "CREATE TABLE 'music' ("
        "'artist' VARCHAR(255) NOT NULL, 'title' VARCHAR(255) NOT NULL,"
        "'last' TIMESTAMP NOT NULL);";

    sqlite_exec(db, create_table_query.c_str(), NULL, NULL, &errmsg);
    handle_error(create_table_query);

    create_table_query = "CREATE TABLE 'unidentified' ("
        "'path' VARCHAR(4096) NOT NULL, 'last' TIMESTAMP NOT NULL,"
        "UNIQUE('path'));";

    sqlite_exec(db, create_table_query.c_str(), NULL, NULL, &errmsg);
    handle_error(create_table_query);
}

int SqlDb::check_artist(const string &artist)
{
    string where_clause = "WHERE artist = '" + artist + "'";
    string query_select = "SELECT * FROM 'music'" + where_clause + ";";

    sqlite_get_table(db, query_select.c_str(), &resultp,
            &nrow, &ncol, &errmsg);
    handle_error(query_select);

    sqlite_free_table(resultp);
    
    return nrow;
}

int SqlDb::check_title(const string &artist, const string &title)
{
    string query_select = "SELECT * FROM 'music' WHERE artist = '"
        + artist + "' AND title = '" + title + "';";

    sqlite_get_table(db, query_select.c_str(), &resultp,
            &nrow, &ncol, &errmsg);
    handle_error(query_select);

    sqlite_free_table(resultp);
    
    return nrow;
}

int SqlDb::check_path(const string &path)
{
    string where_clause = "WHERE path = '" + escape_string(path) + "'";
    string query_select = "SELECT * FROM 'unidentified'" + where_clause + ";";

    sqlite_get_table(db, query_select.c_str(), &resultp,
            &nrow, &ncol, &errmsg);
    handle_error(query_select);

    sqlite_free_table(resultp);
    
    return nrow;
}

time_t SqlDb::get_last(const string &artist, const string &title)
{
    string query_select = "SELECT last FROM 'music' WHERE artist = '"
        + artist + "' AND title = '" + title + "';";

    sqlite_get_table(db, query_select.c_str(), &resultp,
            &nrow, &ncol, &errmsg);
    handle_error(query_select);

    time_t res = ncol ? atol(resultp[1]) : 0;

    sqlite_free_table(resultp);
    
    return res;
}

time_t SqlDb::get_last(const string &path)
{
    string query_select =
        "SELECT last FROM 'unidentified' WHERE path = '" +
        escape_string(path) + "';";

    sqlite_get_table(db, query_select.c_str(), &resultp,
            &nrow, &ncol, &errmsg);
    handle_error(query_select);

    time_t res = ncol ? atol(resultp[1]) : 0;

    sqlite_free_table(resultp);
    
    return res;
}

void SqlDb::update_last(const string &path, time_t last)
{
    check_path(path);

    if (nrow)
    {
        if (last)
        {
            string update_query = "UPDATE 'unidentified' SET last = '"
                + itos(last) + "' WHERE path = '" + escape_string(path) + "';";

            sqlite_exec(db, update_query.c_str(), NULL, NULL, &errmsg);
            handle_error(update_query);
        }
        return;
    }

    string add_query = string("INSERT INTO 'unidentified' ('path', 'last')"
        "VALUES ('") + escape_string(path) + "', '" + itos(last) + "');";

    sqlite_exec(db, add_query.c_str(), NULL, NULL, &errmsg);
    handle_error(add_query);
}

void SqlDb::update_last(const string &artist, const string &title, time_t last)
{
    check_title(artist, title);

    if (nrow)
    {
        if (last)
        {
            string update_query = "UPDATE 'music' SET last = '"
                + itos(last) + "' WHERE artist = '" + artist +
                "' AND title = '" + title + "';";

            sqlite_exec(db, update_query.c_str(), NULL, NULL, &errmsg);
            handle_error(update_query);
        }
        return;
    }

    string add_query = string("INSERT INTO 'music' ('artist', 'title', 'last')"
        "VALUES ('") + artist + "', '" + title + "', '" + itos(last) + "');";

    sqlite_exec(db, add_query.c_str(), NULL, NULL, &errmsg);
    handle_error(add_query);
}

void SqlDb::handle_error(const string &query)
{
    if (errmsg && !strstr(errmsg, "exists"))
    {
        cerr << errmsg << endl;
        cerr << "while executing: " << query << endl;
    }
    free(errmsg);
    errmsg = 0;
}

// Double up single quotes to escape them sqlite style
string SqlDb::escape_string(const string &es)
{
    string s = es;
    unsigned int pos = 0;
    while (1)
    {
        pos = s.find('\'', pos);
        if (pos == string::npos)
            break;
        s.insert(pos, 1, '\'');
        pos += 2;
    }
    return s;
}
